import os

import pytest

from tests.test_data.sender_id import (
    fetch_payload as fetch_sender_id_payload,
    request_payload as request_sender_id_payload,
)
from tests.test_data.messaging import data as messaging_response
from tests.test_data.campaign import get_phonebook
from termii.utils import requests


@pytest.fixture()
def test_environments():
    """
    Add required variables to environment.
    """
    os.environ["TERMII_API_KEY"] = "test123"
    os.environ["TERMII_ENDPOINT_URL"] = "https://api.ng.termii.com"
    os.environ["TERMII_SENDER_ID"] = "test"


class MockFetchSenderIDResponse:
    status_code = 200

    @staticmethod
    def json():
        return fetch_sender_id_payload


@pytest.fixture()
def mock_fetch_sender_id(monkeypatch):
    """Requests.get() mocked to return custom response."""

    def mock_get(*args, **kwargs):
        return MockFetchSenderIDResponse()

    monkeypatch.setattr(requests, "get", mock_get)


class MockRequestSenderIDResponse:
    status_code = 200

    @staticmethod
    def json():
        return request_sender_id_payload


@pytest.fixture()
def mock_request_sender_id(monkeypatch):
    """Requests.get() mocked to return custom response."""

    def mock_post(*args, **kwargs):
        return MockRequestSenderIDResponse()

    monkeypatch.setattr(requests, "post", mock_post)


class MockMessageResponse:
    status_code = 200

    @staticmethod
    def json():
        return messaging_response


@pytest.fixture()
def mock_send_messaging(monkeypatch):
    def mock_post(*args, **kwargs):
        return MockMessageResponse()

    monkeypatch.setattr(requests, "post", mock_post)


class MockGetPhonebookResponse:
    status_code = 200

    @staticmethod
    def json():
        return get_phonebook


@pytest.fixture()
def mock_get_phonebook(monkeypatch):
    def mock_get(*args, **kwargs):
        return MockGetPhonebookResponse()

    monkeypatch.setattr(requests, "get", mock_get)


class MockCreatePhonebookResponse:
    status_code = 200

    @staticmethod
    def json():
        return {"message": "Phonebook added successfully"}


@pytest.fixture()
def mock_create_phonebook(monkeypatch):
    def mock_post(*args, **kwargs):
        return MockCreatePhonebookResponse()

    monkeypatch.setattr(requests, "post", mock_post)


class MockUpdatePhonebookResponse:
    status_code = 200

    @staticmethod
    def json():
        return {"message": "Phonebook updated successfully"}


@pytest.fixture()
def mock_update_phonebook(monkeypatch):
    def mock_patch(*args, **kwargs):
        return MockUpdatePhonebookResponse()

    monkeypatch.setattr(requests, "patch", mock_patch)


class MockDeletePhonebookResponse:
    status_code = 200

    @staticmethod
    def json():
        return {"message": "Phonebook deleted successfully"}


@pytest.fixture()
def mock_delete_phonebook(monkeypatch):
    def mock_delete(*args, **kwargs):
        return MockDeletePhonebookResponse()

    monkeypatch.setattr(requests, "delete", mock_delete)



from typing import Dict
import random
import string

import requests

from termii.schemas.shared import RequestData, RequestType


def make_request(data: RequestData) -> Dict:
    """Handle requests dispatch."""
    match data.type:
        case RequestType.post:
            response = requests.post(
                url=data.url, json=data.payload if data.payload else {}
            )
            return response.json()

        case RequestType.get:
            response = requests.get(url=data.url)
            return response.json()

        case RequestType.patch:
            response = requests.patch(
                url=data.url, json=data.payload if data.payload else {}
            )
            return response.json()

        case RequestType.delete:
            response = requests.delete(url=data.url)
            return response.json()


def get_random_string(count) -> str:
    """Generate random strings."""
    characters = string.ascii_letters + string.digits
    return "".join(random.choice(characters) for _ in range(count))


def get_random_numbers(count) -> str:
    characters = string.digits
    return "".join(random.choice(characters) for _ in range(count))



from enum import Enum


class MessagingChannel(str, Enum):
    generic = "generic"
    dnd = "dnd"
    whatsapp = "whatsapp"


class MessageDistributionType(str, Enum):
    simple = "simple"
    bulk = "bulk"



from typing import Dict, Optional, List

from termii.auth import Client
from termii.schemas.messaging import (
    MessagingChannel,
    MessageDistributionType,
)
from termii.endpoints.messaging import (
    FETCH_SENDER_ID,
    REQUEST_SENDER_ID,
    SEND_MESSAGE,
    SEND_BULK_MESSAGE,
    SEND_FROM_AUTOGENERATED,
)
from termii.schemas.shared import (
    RequestData,
    RequestType,
)
from termii.utils import make_request


class Messaging(Client):
    """
    Base class for core messaging APIs.
    """

    def fetch_sender_id(self, page: Optional[int] = 1) -> Dict:
        """
        `Documentation`: https://developers.termii.com/sender-id

        Fetch sender IDs from API.

        Args:
            `page` (int, None): Page to fetch. Starts from 1.

        Returns:
            Dict.
        """
        self.validate_authentication()

        # Prepare request.
        data = RequestData(
            url=FETCH_SENDER_ID.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL,
                TERMII_API_KEY=self.TERMII_API_KEY,
                data_page=page,
            ),
            type=RequestType.get,
        )
        return make_request(data=data)

    def request_sender_id(
        self,
        sender_id: str,
        usecase: str,
        company: str,
        api_key: str,
        endpoint_url: str,
    ) -> Dict:
        """
        `Documentation`: https://developers.termii.com/sender-id

        Request new sender ID from API.

        Args:
            `sender_id` (str): Represents the ID of the sender which can be
            alphanumeric or numeric. Alphanumeric sender ID length
            should be between 3 and 11 characters (Example:CompanyName)

            `usecase` (str): A sample of the type of message sent.

            `company` (str): Represents the name of the company with the
                            sender ID.

            `api_key` (str): API key from Termii dashboard.

            `endpoint_url` (str): Endpoint url from dashboard.
            E.g https://api.ng.termii.com

        Returns:
            Dict.
        """
        payload = dict(
            api_key=api_key,
            sender_id=sender_id,
            usecase=usecase,
            company=company,
        )
        request_data = RequestData(
            url=REQUEST_SENDER_ID.format(TERMII_ENDPOINT_URL=endpoint_url),
            payload=payload,
            type=RequestType.post,
        )
        return make_request(data=request_data)

    def send_message(
        self,
        receivers: List[str],
        text: str,
        media_url: Optional[str] = None,
        media_caption: Optional[str] = None,
        channel: Optional[MessagingChannel] = MessagingChannel.generic,
        distribution_type: Optional[
            MessageDistributionType
        ] = MessageDistributionType.simple,
    ) -> Dict:
        """
        `Documentation`: https://developers.termii.com/messaging-api

        Send text messages to their customers across different messaging channels.

        Args:
            `receivers` (List[str]): Accepts a list of receipient phone numbers in
            international format. An example is [2348364528908]. Note the absence of
            +.

            `channel` (MessagingChannel): This is the route through which the message is sent.
            It is either `MessagingChannel.dnd`, `MessagingChannel.whatsapp`,
            or `MessagingChannel.generic`.

            `media` (Optional[str]): This is a media object, it is only available for
            `MessagingChannel.whatsapp` channel.

            `media_url` (Optional[str]): The url to the file resource. Only the following
            formats are supported: JPG, JPEG, PNG, MP3, OGG, AMR, PDF, MP4

            `media_caption` (Optional[str]): The caption that should be added to the media.

            `text` (str): Text of a message that would be sent to the destination phone number.

            `distribution_type` (MessageDistributionType): This defines the volume of
            receivers allowed. Van either be MessageDistributionType.simple or
            MessageDistributionType.bulk

        Returns:
            Dict
        """  # noqa

        # Data validation.
        self.validate_authentication()
        if media_url:
            if channel != MessagingChannel.whatsapp:
                raise ValueError(
                    f"Media messages are only allowed with {MessagingChannel.whatsapp.value} channel."  # noqa
                )

        if distribution_type.value == MessageDistributionType.simple:
            if len(receivers) >= 100:
                raise ValueError(
                    "You cannot send to more than 100 mobile numbers."
                )
        else:
            if len(receivers) >= 10000:
                raise ValueError(
                    "You cannot send to more than 10000 mobile numbers."
                )

        # Prepare payload.
        payload = {}
        payload["api_key"] = self.TERMII_API_KEY
        payload["from"] = self.TERMII_SENDER_ID
        payload["to"] = receivers
        payload["type"] = "plain"
        payload["channel"] = channel.value
        payload["sms"] = text

        if media_url:
            payload["media"] = dict(media_url=media_url, caption=media_caption)

        # Make request.
        request_url = ""
        if distribution_type.value == MessageDistributionType.simple:
            request_url = SEND_MESSAGE.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL
            )
        else:
            request_url = SEND_BULK_MESSAGE.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL
            )

        request_data = RequestData(
            url=request_url,
            payload=payload,
            type=RequestType.post,
        )
        return make_request(data=request_data)

    def send_message_from_autogenerated_number(
        self,
        receivers: List[str],
        text: str,
    ) -> Dict:
        """
        `Documentation`: https://developer.termii.com/number

        This API allows businesses send messages to customers using Termii's
        auto-generated messaging numbers that adapt to customers location.

        Args:
            `receivers` (List[str]): Accepts a list of receipient phone numbers in
            international format. An example is [2348364528908]. Note the absence of
            +.

            `text` (str): Text of a message that would be sent to the destination phone number

        Returns:
            Dict
        """  # noqa

        # Data validation.
        self.validate_authentication()
        if len(receivers) >= 100:
            raise ValueError(
                "You cannot send to more than 100 mobile numbers."
            )

        # Prepare payload.
        payload = {}
        payload["api_key"] = self.TERMII_API_KEY
        payload["to"] = receivers
        payload["sms"] = text

        request_data = RequestData(
            url=SEND_FROM_AUTOGENERATED.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL
            ),
            payload=payload,
            type=RequestType.post,
        )
        return make_request(data=request_data)



import os


class Client:
    def authenticate_from_env(self) -> None:
        """
        Authenticate requests using variables in the environment.
        Method expects the following variables to be present in the environment:
            `TERMII_API_KEY`, `TERMII_ENDPOINT_URL`, `TERMII_SENDER_ID`

        Raises:
            `ValueError`: If any of the expected environment variables are not present.
        """  # noqa

        # Get variables.
        environment = os.environ
        self.TERMII_API_KEY = environment.get("TERMII_API_KEY")
        self.TERMII_ENDPOINT_URL = environment.get("TERMII_ENDPOINT_URL")
        self.TERMII_SENDER_ID = environment.get("TERMII_SENDER_ID")

        # Validate variables.
        if self.TERMII_API_KEY is None:
            raise ValueError(
                "TERMII_API_KEY is not present in the environment."
            )
        if self.TERMII_ENDPOINT_URL is None:
            raise ValueError(
                "TERMII_ENDPOINT_URL is not present in the environment."
            )
        if self.TERMII_SENDER_ID is None:
            raise ValueError(
                "TERMII_SENDER_ID is not present in the environment."
            )

    def authenticate_direct(
        self, api_key: str, endpoint_url: str, sender_id: str
    ) -> None:
        """
        Directly authenticate with entered values.

        Args:
            `api_key` (str): API key from Termii dashboard.

            `endpoint_url` (str): Endpoint url from Termii dashboard.

            `sender_id` (str): Sender ID from Termii dashboard.
        """
        self.TERMII_API_KEY = api_key
        self.TERMII_ENDPOINT_URL = endpoint_url
        self.TERMII_SENDER_ID = sender_id

    def validate_authentication(self) -> None:
        """
        Ensure that API keys are provided.
        """
        if (
            not hasattr(self, "TERMII_API_KEY")
            or not hasattr(self, "TERMII_ENDPOINT_URL")
            or not hasattr(self, "TERMII_SENDER_ID")
        ):
            raise ValueError("Authentication required.")



from typing import Optional, Dict
from enum import Enum

from pydantic import BaseModel


class RequestType(str, Enum):
    post = "post"
    get = "get"
    patch = "patch"
    delete = "delete"


class RequestData(BaseModel):
    url: str
    payload: Optional[Dict] = None
    type: RequestType



# flake8: noqa


fetch_payload = {
    "current_page": 1,
    "data": [
        {
            "sender_id": "ACME Key",
            "status": "unblock",
            "company": "ACME",
            "usecase": None,
            "country": None,
            "created_at": "2021-03-29 16:51:53",
        },
        {
            "sender_id": "ACME Alert",
            "status": "unblock",
            "company": "ACME",
            "usecase": None,
            "country": None,
            "created_at": "2021-03-29 16:51:09",
        },
        {
            "sender_id": "ACME Wall",
            "status": "unblock",
            "company": "ACME",
            "usecase": None,
            "country": None,
            "created_at": "2021-03-29 16:50:38",
        },
        {
            "sender_id": "Hooli",
            "status": "unblock",
            "company": "Hooli",
            "usecase": None,
            "country": None,
            "created_at": "2021-03-26 13:19:16",
        },
        {
            "sender_id": "Google",
            "status": "unblock",
            "company": None,
            "usecase": None,
            "country": None,
            "created_at": "2021-01-19 17:41:30",
        },
        {
            "sender_id": "Facebook",
            "status": "unblock",
            "company": None,
            "usecase": None,
            "country": None,
            "created_at": "2021-01-19 17:41:14",
        },
    ],
    "first_page_url": "https://api.ng.termii.com/api/sender-id?page=1",
    "from": 1,
    "last_page": 47,
    "last_page_url": "https://api.ng.termii.com/api/sender-id?page=47",
    "next_page_url": "https://api.ng.termii.com/api/sender-id?page=2",
    "path": "https://api.ng.termii.com/api/sender-id",
    "per_page": 10,
    "prev_page_url": None,
    "to": 15,
    "total": 704,
}


request_payload = {
    "code": "ok",
    "message": "Sender Id requested. You will be  contacted by your account manager.",
}



import os

import pytest
from termii.auth import Client


class TestClient:
    def test_authenticate_behaviour_if_no_api_key(self, test_environments):
        """
        Test authentication behaviour if no TERMII_API_KEY is added
        to the environment.
        """
        del os.environ["TERMII_API_KEY"]

        termii_client = Client()
        with pytest.raises(ValueError) as value_error:
            termii_client.authenticate_from_env()
            assert (
                str(value_error.value)
                == "TERMII_API_KEY is not present in the environment."
            )

    def test_authenticate_behaviour_if_no_endpoint_url(
        self, test_environments
    ):
        """
        Test authentication behaviour if no TERMII_ENDPOINT_URL
        is added to the environment.
        """
        del os.environ["TERMII_ENDPOINT_URL"]

        termii_client = Client()
        with pytest.raises(ValueError) as value_error:
            termii_client.authenticate_from_env()
        assert (
            str(value_error.value)
            == "TERMII_ENDPOINT_URL is not present in the environment."
        )

    def test_authenticate_behaviour_if_no_sender_id(self, test_environments):
        """
        Test authentication behaviour if no TERMII_SENDER_ID is
        added to the environment.
        """
        del os.environ["TERMII_SENDER_ID"]

        termii_client = Client()
        with pytest.raises(ValueError) as value_error:
            termii_client.authenticate_from_env()
        assert (
            str(value_error.value)
            == "TERMII_SENDER_ID is not present in the environment."
        )

    def test_authenticate_client_by_env(self, test_environments):
        """Test that env authentication works."""
        termii_client = Client()
        termii_client.authenticate_from_env()
        assert termii_client.TERMII_API_KEY == os.environ["TERMII_API_KEY"]
        assert (
            termii_client.TERMII_ENDPOINT_URL
            == os.environ["TERMII_ENDPOINT_URL"]
        )
        assert termii_client.TERMII_SENDER_ID == os.environ["TERMII_SENDER_ID"]

    def test_authenticate_direct(self, test_environments):
        """Test that direct authentication."""

        api_key = "test_key"
        endpoint_url = "https://api.ng.termii.com"
        sender_id = "test_id"

        termii_client = Client()
        termii_client.authenticate_direct(
            api_key=api_key, endpoint_url=endpoint_url, sender_id=sender_id
        )

        assert termii_client.TERMII_API_KEY == api_key
        assert termii_client.TERMII_ENDPOINT_URL == endpoint_url
        assert termii_client.TERMII_SENDER_ID == sender_id



FETCH_PHONEBOOKS = (
    "{TERMII_ENDPOINT_URL}/api/phonebooks?api_key={TERMII_API_KEY}&page={page}"
)
CREATE_PHONEBOOK = "{TERMII_ENDPOINT_URL}/api/phonebooks"
UPDATE_PHONEBOOK = "{TERMII_ENDPOINT_URL}/api/phonebooks/{phonebook_id}"
DELETE_PHONEBOOK = (
    "{TERMII_ENDPOINT_URL}/api/phonebooks/{phonebook_id}?api_key={TERMII_API_KEY}"  # noqa
)



FETCH_SENDER_ID = (
    "{TERMII_ENDPOINT_URL}/sender-id?api_key={TERMII_API_KEY}&page={data_page}"
)
REQUEST_SENDER_ID = "{TERMII_ENDPOINT_URL}/api/sender-id/request"
SEND_MESSAGE = "{TERMII_ENDPOINT_URL}/api/sms/send"
SEND_BULK_MESSAGE = "{TERMII_ENDPOINT_URL}/api/sms/send/bulk"
SEND_FROM_AUTOGENERATED = "{TERMII_ENDPOINT_URL}/api/sms/number/send"



from typing import Dict, Optional

from termii.auth import Client

from termii.schemas.shared import (
    RequestData,
    RequestType,
)
from termii.utils import make_request
from termii.endpoints.campaign import (
    FETCH_PHONEBOOKS,
    CREATE_PHONEBOOK,
    UPDATE_PHONEBOOK,
    DELETE_PHONEBOOK,
)


class Campaign(Client):
    """
    Handle operations related to phonebook.
    """

    def get_phonebooks(self, page: Optional[int] = 1) -> Dict:
        """
        Get all saved phonebooks.

        Args:
            `page` (int): Page to return. Starts at 1.
        """

        self.validate_authentication()
        data = RequestData(
            url=FETCH_PHONEBOOKS.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL,
                TERMII_API_KEY=self.TERMII_API_KEY,
                page=page,
            ),
            type=RequestType.get,
        )
        return make_request(data=data)

    def create_phonebook(self, name: str, description: str) -> Dict:
        """
        Create a phonebook.

        Args:
            `name` (str): The name of the phonebook.

            `description` (str): A description of the contacts stored in the phonebook.
        """  # noqa
        self.validate_authentication()
        payload = dict(
            api_key=self.TERMII_API_KEY,
            phonebook_name=name,
            description=description,
        )
        request_data = RequestData(
            url=CREATE_PHONEBOOK.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL,
            ),
            payload=payload,
            type=RequestType.post,
        )
        return make_request(data=request_data)

    def update_phonebook(
        self, name: str, description: str, phonebook_id: str
    ) -> Dict:
        """
        Update a phonebook.

        Args:
            `name` (str): The name of the phonebook.

            `description` (str): A description of the contacts stored in the phonebook.

            `phonebook_id` (str): ID of phonebook to update.
        """  # noqa
        self.validate_authentication()
        payload = dict(
            api_key=self.TERMII_API_KEY,
            phonebook_name=name,
            description=description,
        )
        request_data = RequestData(
            url=UPDATE_PHONEBOOK.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL,
                phonebook_id=phonebook_id,
            ),
            payload=payload,
            type=RequestType.patch,
        )
        return make_request(data=request_data)

    def delete_phonebook(self, phonebook_id: str) -> Dict:
        """
        Delete a phonebook.

        Args:

            `phonebook_id` (str): ID of phonebook to update.
        """
        self.validate_authentication()
        request_data = RequestData(
            url=DELETE_PHONEBOOK.format(
                TERMII_ENDPOINT_URL=self.TERMII_ENDPOINT_URL,
                phonebook_id=phonebook_id,
                TERMII_API_KEY=self.TERMII_API_KEY,
            ),
            type=RequestType.delete,
        )
        return make_request(data=request_data)



import uuid

from termii.campaign import Campaign
from tests.test_data.campaign import get_phonebook


class TestPhonebook:
    def test_get_phonebooks(self, test_environments, mock_get_phonebook):
        """Test fetch of phonebooks"""
        campaign_client = Campaign()
        campaign_client.authenticate_from_env()
        phonebooks = campaign_client.get_phonebooks()

        assert len(phonebooks.get("data")) == len(get_phonebook.get("data"))
        assert (
            phonebooks.get("data")[0]["id"]
            == get_phonebook.get("data")[0]["id"]
        )
        assert (
            phonebooks.get("data")[0]["name"]
            == get_phonebook.get("data")[0]["name"]
        )
        assert (
            phonebooks.get("data")[0]["last_updated"]
            == get_phonebook.get("data")[0]["last_updated"]
        )

    def test_create_phonebook(self, test_environments, mock_create_phonebook):
        """Create phonebook."""
        campaign_client = Campaign()
        campaign_client.authenticate_from_env()
        response = campaign_client.create_phonebook(
            name="Test phonebook", description="Test description."
        )
        response.get("message") == "Phonebook added successfully"

    def test_update_phonebook(self, test_environments, mock_update_phonebook):
        """Test Update phonebook."""
        campaign_client = Campaign()
        campaign_client.authenticate_from_env()
        response = campaign_client.update_phonebook(
            name="Test phonebook",
            description="Test description.",
            phonebook_id=str(uuid.uuid4()),
        )
        response.get("message") == "Phonebook updated successfully"

    def test_delete_phonebook(self, test_environments, mock_delete_phonebook):
        """Test delete phonebook."""
        campaign_client = Campaign()
        campaign_client.authenticate_from_env()
        data = campaign_client.delete_phonebook(phonebook_id=str(uuid.uuid4()))
        assert data.get("message") == "Phonebook deleted successfully"



import os

import pytest

from termii.messaging import Messaging
from tests.test_data.sender_id import fetch_payload as fetch_sender_id_payload
from termii.schemas.messaging import (
    MessagingChannel,
    MessageDistributionType,
)
from termii.utils import get_random_numbers


class TestSenderID:
    def test_fetch_sender_id(self, test_environments, mock_fetch_sender_id):
        """Test fetch of sender id"""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()
        sender_ids = messaging_client.fetch_sender_id(page=1)
        assert sender_ids is not None

        print(sender_ids)
        assert len(sender_ids["data"]) == len(fetch_sender_id_payload["data"])

    def test_fetch_sender_id_without_auth(
        self, test_environments, mock_fetch_sender_id
    ):
        """Test fetch of sender ID without authentication"""
        del os.environ["TERMII_API_KEY"]
        del os.environ["TERMII_ENDPOINT_URL"]
        del os.environ["TERMII_SENDER_ID"]

        messaging_client = Messaging()
        with pytest.raises(ValueError) as value_error:
            messaging_client.fetch_sender_id(page=1)

        assert str(value_error.value) == "Authentication required."

    def test_request_sender_id(self, mock_request_sender_id):
        messaging_client = Messaging()
        result = messaging_client.request_sender_id(
            sender_id="test id",
            usecase="test use case",
            company="test company",
            api_key="Test key",
            endpoint_url="https://api.ng.termii.com",
        )
        assert result is not None
        assert result.get("code") == "ok"
        assert (
            result.get("message")
            == "Sender Id requested. You will be  contacted by your account manager."  # noqa
        )


class TestSendSimpleMessage:
    def test_send_1(self, test_environments, mock_send_messaging):
        """Test send of simple messages."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(50)]
        response = messaging_client.send_message(
            receivers=receivers, text="Hello all. Thanks for visiting."
        )
        assert response["message"] == "Successfully Sent"

    def test_send_max_receipients(
        self, test_environments, mock_send_messaging
    ):
        """Test send of simple messages if recepients are beyond max limit."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(500)]
        with pytest.raises(ValueError) as value_error:
            messaging_client.send_message(
                receivers=receivers, text="Hello all. Thanks for visiting."
            )
        assert (
            str(value_error.value)
            == "You cannot send to more than 100 mobile numbers."
        )

    def test_send_bulk_max_receipients(
        self, test_environments, mock_send_messaging
    ):
        """Test send of bulk messages if recepients are beyond max limit."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(20000)]
        with pytest.raises(ValueError) as value_error:
            messaging_client.send_message(
                receivers=receivers,
                text="Hello all. Thanks for visiting.",
                distribution_type=MessageDistributionType.bulk,
            )
        assert (
            str(value_error.value)
            == "You cannot send to more than 10000 mobile numbers."
        )

    def test_send_simple_message_with_media(
        self, test_environments, mock_send_messaging
    ):
        """Test behaviour if simple message has a media."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(20000)]
        with pytest.raises(ValueError) as value_error:
            messaging_client.send_message(
                receivers=receivers,
                text="Hello all. Thanks for visiting.",
                distribution_type=MessageDistributionType.bulk,
                media_url="http://test.com/hello.jpeg",
            )
        assert (
            str(value_error.value)
            == "Media messages are only allowed with whatsapp channel."
        )

    def test_send_bulk(self, test_environments, mock_send_messaging):
        """Test send of bulk messages."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(50)]
        response = messaging_client.send_message(
            receivers=receivers,
            text="Hello all. Thanks for visiting.",
            distribution_type=MessageDistributionType.bulk,
        )
        assert response["message"] == "Successfully Sent"

    def test_send_whatsapp_bulk(self, test_environments, mock_send_messaging):
        """Test send of bulk whatsapp messages."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(50)]
        response = messaging_client.send_message(
            receivers=receivers,
            text="Hello all. Thanks for visiting.",
            distribution_type=MessageDistributionType.bulk,
            channel=MessagingChannel.whatsapp,
        )
        assert response["message"] == "Successfully Sent"

    def test_send_from_autogenerated_number(
        self, test_environments, mock_send_messaging
    ):
        """Test send message from autogenerated number."""
        messaging_client = Messaging()
        messaging_client.authenticate_from_env()

        receivers = [get_random_numbers(13) for _ in range(50)]
        response = messaging_client.send_message_from_autogenerated_number(
            receivers=receivers,
            text="Hello all. Thanks for visiting.",
        )
        assert response["message"] == "Successfully Sent"
